%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%% Example Simulation %%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
% Please install Matlab-software by Mathworks, DirectLiNGAM-algorithm by 
% Shimizu et al. 
% (http://www.ar.sanken.osaka-u.ac.jp/~sshimizu/code/Dlingamcode.html), 
% and additional code required by the algorithm
% (available from http://www.di.ens.fr/~fbach/kernel-ica/index.htm). 
% Add the installed folders to Matlab-path, and then type "simulation_demo"
% to matlab command line in this folder initiating a short example of 
% manuscript simulations, or examine "simulation_demo.m" file in a text 
% editor. You will need Matlab's Statistics toolbox/library.


% load data
disp('Loading the data on mBDI and Sleep problems');
load mBDIdataset.dat;
dep = mBDIdataset(:,5); nukvaik = mBDIdataset(:,4);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% fit simulation distributions to real data and plot results %%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('Fitting and plotting simulation distributions');
r = regstats(dep,nukvaik);
options = statset('MaxIter',1000);
gmfit = gmdistribution.fit(r.r,4,'Options',options);
gmfitd = gmdistribution.fit(dep,4,'Options',options);
a=expfit(nukvaik-1);

figure; 
subplot(1,3,1);
edg = linspace(min(nukvaik),max(nukvaik),20); [f,xi] = ksdensity(nukvaik-1);
n = histc(nukvaik,edg); bar(edg,n,'histc'); colormap gray;
set(get(gca,'Children'),'FaceColor',[0.5 0.5 0.5]); hold;
plot(xi'+1,(edg(20)-edg(1))*length(dep)/19*exppdf(xi',a),'k','LineWidth',1.5);
plot(xi+1,(edg(20)-edg(1))*length(dep)/19*f,'k','LineWidth',0.5);
set(gca,'FontSize',10);
ylabel('Number of observations');
legend('Sleep problems','Model density','Kernel smoother');
title('Independent variable');
annotation('textbox',[0.1, 0.87, 0.1, 0.1],'String','A',...
    'FontSize',12,'LineStyle','none','FontWeight','bold');

subplot(1,3,2);
edg = linspace(min(r.r),max(r.r),20); [f,xi] = ksdensity(r.r);
n = histc(r.r,edg); bar(edg,n,'histc'); colormap gray;
set(get(gca,'Children'),'FaceColor',[0.5 0.5 0.5]); hold; 
plot(xi',(edg(20)-edg(1))*length(r.r)/19*pdf(gmfit,xi'),'k','LineWidth',1.5);
plot(xi,(edg(20)-edg(1))*length(r.r)/19*f,'k','LineWidth',0.5);
set(gca,'FontSize',10);
ylabel('Number of observations');
legend('Residual data','GM density','Kernel smoother'); %legend('boxoff');
title('Linear model residual');
annotation('textbox',[0.39, 0.87, 0.1, 0.1],'String','B',...
    'FontSize',12,'LineStyle','none','FontWeight','bold');

subplot(1,3,3);
edg = linspace(min(dep),max(dep),20); [f,xi] = ksdensity(dep);
n = histc(dep,edg); bar(edg,n,'histc'); colormap gray;
set(get(gca,'Children'),'FaceColor',[0.5 0.5 0.5]); hold;
plot(xi',(edg(20)-edg(1))*length(dep)/19*pdf(gmfitd,xi'),'k','LineWidth',1.5);
plot(xi,(edg(20)-edg(1))*length(dep)/19*f,'k','LineWidth',0.5);
set(gca,'FontSize',10);
ylabel('Number of observations');
legend('mBDI data','GM density','Kernel smoother');
title('Dependent variable');
annotation('textbox',[0.67, 0.87, 0.1, 0.1],'String','C',...
    'FontSize',12,'LineStyle','none','FontWeight','bold');
drawnow;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%% Simulate confounding and plot results %%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('Simulating performance for 15 different degrees of confounding');
disp('This may take some time...15 lambda-values');

tic;
lambda = linspace(0,1,15); n = length(r.r);
kkICA = zeros(1000,length(lambda)); %bt = zeros(1000,length(lambda));
kskew = zeros(1000,length(lambda));
ktanh = zeros(1000,length(lambda));
for j = 1:length(lambda)
    disp(['lambda is ', num2str(lambda(j))]);
    for i = 1:1000
        try
            e = random(gmfit,n);
            e2 = random(gmfit,n);
            e = e - mean(e); e2 = e2 - mean(e2);
            x = exprnd(a,n,1)+1;
            z = exprnd(a,n,1)+1;
            y = (1-lambda(j))*r.beta(2)*x + e + lambda(j)*r.beta(2)*z;
            x = (1-lambda(j))*x + lambda(j)*(r.beta(2)*z + e2);
            
%             [B, stde, ci, k] = Dlingam([y';x']);
            JkICA = bcaBootfunc([x,y],'kICA');
            Jskew = bcaBootfunc([x,y],'skew');
            Jtanh = bcaBootfunc([x,y],'tanh');
        catch em
            e = random(gmfit,n);
            e2 = random(gmfit,n);
            e = e - mean(e); e2 = e2 - mean(e2);
            x = exprnd(a,n,1)+1;
            z = exprnd(a,n,1)+1;
            y = (1-lambda(j))*r.beta(2)*x + e + lambda(j)*r.beta(2)*z;
            x = (1-lambda(j))*x + lambda(j)*(r.beta(2)*z + e2);
            
%             [B, stde, ci, k] = Dlingam([y';x']);
            JkICA = bcaBootfunc([x,y],'kICA');
            Jskew = bcaBootfunc([x,y],'skew');
            Jtanh = bcaBootfunc([x,y],'tanh');
        end
        
        kkICA(i,j) = JkICA; %bt(i,j) = B(1,2);
        kskew(i,j) = Jskew;
        ktanh(i,j) = Jtanh;
    end
end
toc

figure; subplot(1,3,1);
plot(lambda,sum(kkICA>0)/1000,'ko','LineWidth',1.5); hold;
plot(lambda,sum(kkICA>0)/1000,'k','LineWidth',1.5); title('DirectLiNGAM');
ylabel('Estimation success'); xlabel('Degree of confounding');
subplot(1,3,2);
plot(lambda,sum(kskew>0)/1000,'ko','LineWidth',1.5); hold;
plot(lambda,sum(kskew>0)/1000,'k','LineWidth',1.5); title('Skew-based');
xlabel('Degree of confounding');
subplot(1,3,3); 
plot(lambda,sum(ktanh>0)/1000,'ko','LineWidth',1.5); hold;
plot(lambda,sum(ktanh>0)/1000,'k','LineWidth',1.5); title('Tanh-based');
xlabel('Degree of confounding');
suptitle('Linear confounding test');
drawnow;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%% Compute a real data bootstrap-result %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('Computing 2000 bootstrap estimates of causality for real data');
tic;
kkICA2 = zeros(2000,1);
kskew2 = zeros(2000,1);
ktanh2 = zeros(2000,1);
for i =  1:2000
%     [B, stde, ci, k] = Dlingam(boot([dep,nukvaik])');
    kkICA2(i) = bcaBootfunc(boot([dep,nukvaik]),'kICA');
    kskew2(i) = bcaBootfunc(boot([dep,nukvaik]),'skew');
    ktanh2(i) = bcaBootfunc(boot([dep,nukvaik]),'tanh');
end
toc

certainty = sum(kkICA2<0)/2000;
disp(['Sleep problems was causal antecedent by DirectLiNGAM in ',num2str(100*certainty),...
    ' percent of bootstrap re-samples']);
certainty = sum(kskew2<0)/2000;
disp(['Sleep problems was causal antecedent by Skew-based estimate in ',num2str(100*certainty),...
    ' percent of bootstrap re-samples']);
certainty = sum(ktanh2<0)/2000;
disp(['Sleep problems was causal antecedent by Tanh-based estimate in ',num2str(100*certainty),...
    ' percent of bootstrap re-samples']);
% disp(['Regression cofficient over these was ',num2str(mean(b2(ks==2))),...
%     ' (s.d. ',num2str(std(b2(ks==2))),').']);

disp('The demo ends here, thank you.');